"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.print = void 0;
const chalk = require("chalk");
const constants_1 = require("./constants");
const numbers_1 = require("./numbers");
const validate_1 = require("./validate");
const ntext = (text, plural, count) => Math.abs(count) === 1 ? text : plural;
const getMetricUnit = (metric, count) => {
    switch (metric) {
        case "releases":
        case "major":
        case "minor":
        case "patch":
            return ntext("release", "releases", count);
        case "drift":
        case "pulse":
        default:
            return ntext("libyear", "libyears", count);
    }
};
const printIndividual = (violations) => {
    violations.forEach((dependencies, metric) => {
        dependencies.forEach(({ threshold, value }, dependency) => {
            console.error(`${chalk.magenta(metric)}: ${chalk.cyan(dependency)} is ${chalk.red(`${numbers_1.printFloat(value)} ${getMetricUnit(metric, value)}`)} behind; threshold is ${chalk.yellow(numbers_1.printFloat(threshold))}.`);
        });
    });
};
const printCollective = (totals, violations, threshold) => {
    const isBreach = (metric) => violations.has(metric);
    const logger = (metric) => isBreach(metric) ? console.error : console.log;
    const message = (metric, value, limit) => {
        const valueStyler = isBreach(metric) ? chalk.red : chalk.greenBright;
        const valueMessage = `${chalk.magenta(metric)}: ${{
            drift: "package is",
            pulse: "dependencies are",
            releases: "dependencies are",
            major: "dependencies are",
            minor: "dependencies are",
            patch: "dependencies are",
        }[metric]} ${valueStyler(`${numbers_1.printFloat(value)} ${getMetricUnit(metric, value)}`)} behind`;
        const limitMessage = `threshold is ${chalk.yellow(numbers_1.printFloat(limit))}`;
        return isBreach(metric)
            ? `${valueMessage}; ${limitMessage}.`
            : `${valueMessage}.`;
    };
    constants_1.metrics.forEach((metric) => {
        logger(metric)(message(metric, totals.get(metric), threshold === null || threshold === void 0 ? void 0 : threshold[`${metric}Collective`]));
    });
};
const print = (dependencies, threshold, overrides) => {
    console.table(dependencies.map(({ dependency, drift, pulse, releases, major, minor, patch, available, }) => ({
        dependency,
        drift: numbers_1.printFloat(drift),
        pulse: numbers_1.printFloat(pulse),
        releases,
        major,
        minor,
        patch,
        available,
    })));
    console.log();
    const totals = validate_1.getTotals(dependencies);
    const violations = validate_1.getViolations(dependencies, totals, threshold, overrides);
    const hasIndividualViolations = Array.from(violations.individual.values()).reduce((acc, cur) => acc + cur.size, 0) > 0;
    const hasCollectiveViolations = violations.collective.size > 0;
    if (hasIndividualViolations) {
        console.log(chalk.bold("# Individual"));
        printIndividual(violations.individual);
        console.log();
    }
    console.log(chalk.bold("# Collective"));
    printCollective(totals, violations.collective, threshold);
    console.log();
    if (hasIndividualViolations || hasCollectiveViolations) {
        process.exit(1);
    }
};
exports.print = print;
